# -*- coding: utf-8 -*-
# Generated by Django 1.11.24 on 2020-07-16 16:28
from __future__ import unicode_literals

from django.conf import settings
from django.db import migrations, models
import django.db.models.deletion
import django_countries.fields


class Migration(migrations.Migration):
    initial = True

    dependencies = [
        migrations.swappable_dependency(settings.AUTH_USER_MODEL),
    ]

    operations = [
        migrations.SeparateDatabaseAndState(
            state_operations=[
                migrations.CreateModel(
                    name="Class",
                    fields=[
                        (
                            "id",
                            models.AutoField(
                                auto_created=True,
                                primary_key=True,
                                serialize=False,
                                verbose_name="ID",
                            ),
                        ),
                        ("name", models.CharField(max_length=200)),
                        ("access_code", models.CharField(max_length=5)),
                        (
                            "classmates_data_viewable",
                            models.BooleanField(default=False),
                        ),
                        ("always_accept_requests", models.BooleanField(default=False)),
                        ("accept_requests_until", models.DateTimeField(null=True)),
                        (
                            "teacher",
                            models.ForeignKey(
                                on_delete=django.db.models.deletion.CASCADE,
                                related_name="class_teacher",
                                to="common.Teacher",
                            ),
                        ),
                    ],
                    options={
                        "verbose_name_plural": "classes",
                    },
                ),
            ],
            database_operations=[],
        ),
        migrations.SeparateDatabaseAndState(
            state_operations=[
                migrations.CreateModel(
                    name="School",
                    fields=[
                        (
                            "id",
                            models.AutoField(
                                auto_created=True,
                                primary_key=True,
                                serialize=False,
                                verbose_name="ID",
                            ),
                        ),
                        ("name", models.CharField(max_length=200)),
                        ("postcode", models.CharField(max_length=10)),
                        ("town", models.CharField(max_length=200)),
                        ("latitude", models.CharField(max_length=20)),
                        ("longitude", models.CharField(max_length=20)),
                        ("country", django_countries.fields.CountryField(max_length=2)),
                    ],
                    options={
                        "permissions": (
                            (
                                "view_aggregated_data",
                                "Can see available aggregated data",
                            ),
                            (
                                "view_map_data",
                                "Can see schools' location displayed on map",
                            ),
                        ),
                    },
                ),
            ],
            database_operations=[],
        ),
        migrations.SeparateDatabaseAndState(
            state_operations=[
                migrations.CreateModel(
                    name="Student",
                    fields=[
                        (
                            "id",
                            models.AutoField(
                                auto_created=True,
                                primary_key=True,
                                serialize=False,
                                verbose_name="ID",
                            ),
                        ),
                        (
                            "class_field",
                            models.ForeignKey(
                                null=True,
                                on_delete=django.db.models.deletion.CASCADE,
                                related_name="students",
                                to="common.Class",
                            ),
                        ),
                        (
                            "new_user",
                            models.OneToOneField(
                                blank=True,
                                null=True,
                                on_delete=django.db.models.deletion.CASCADE,
                                related_name="new_student",
                                to=settings.AUTH_USER_MODEL,
                            ),
                        ),
                        (
                            "pending_class_request",
                            models.ForeignKey(
                                null=True,
                                on_delete=django.db.models.deletion.SET_NULL,
                                related_name="class_request",
                                to="common.Class",
                            ),
                        ),
                        (
                            "user",
                            models.OneToOneField(
                                on_delete=django.db.models.deletion.CASCADE,
                                to="common.UserProfile",
                            ),
                        ),
                    ],
                ),
            ],
            database_operations=[],
        ),
        migrations.SeparateDatabaseAndState(
            state_operations=[
                migrations.CreateModel(
                    name="Teacher",
                    fields=[
                        (
                            "id",
                            models.AutoField(
                                auto_created=True,
                                primary_key=True,
                                serialize=False,
                                verbose_name="ID",
                            ),
                        ),
                        ("title", models.CharField(max_length=35)),
                        ("is_admin", models.BooleanField(default=False)),
                        (
                            "new_user",
                            models.OneToOneField(
                                blank=True,
                                null=True,
                                on_delete=django.db.models.deletion.CASCADE,
                                related_name="new_teacher",
                                to=settings.AUTH_USER_MODEL,
                            ),
                        ),
                        (
                            "pending_join_request",
                            models.ForeignKey(
                                blank=True,
                                null=True,
                                on_delete=django.db.models.deletion.SET_NULL,
                                related_name="join_request",
                                to="common.School",
                            ),
                        ),
                        (
                            "school",
                            models.ForeignKey(
                                null=True,
                                on_delete=django.db.models.deletion.SET_NULL,
                                related_name="teacher_school",
                                to="common.School",
                            ),
                        ),
                        (
                            "user",
                            models.OneToOneField(
                                on_delete=django.db.models.deletion.CASCADE,
                                to="common.UserProfile",
                            ),
                        ),
                    ],
                ),
            ],
            database_operations=[],
        ),
        migrations.SeparateDatabaseAndState(
            state_operations=[
                migrations.CreateModel(
                    name="UserProfile",
                    fields=[
                        (
                            "id",
                            models.AutoField(
                                auto_created=True,
                                primary_key=True,
                                serialize=False,
                                verbose_name="ID",
                            ),
                        ),
                        (
                            "can_view_aggregated_data",
                            models.BooleanField(default=False),
                        ),
                        ("developer", models.BooleanField(default=False)),
                        (
                            "awaiting_email_verification",
                            models.BooleanField(default=False),
                        ),
                        (
                            "user",
                            models.OneToOneField(
                                on_delete=django.db.models.deletion.CASCADE,
                                to=settings.AUTH_USER_MODEL,
                            ),
                        ),
                    ],
                ),
            ],
            database_operations=[],
        ),
    ]



from django.contrib.auth.mixins import LoginRequiredMixin
from django.contrib.auth.views import redirect_to_login
from django.core.cache import cache
from django.core.exceptions import FieldDoesNotExist
from two_factor.utils import default_device


def two_factor_cache_key(user):
    """Cache key for using_two_factor."""
    return "using-two-factor-%s" % user.pk


def _using_two_factor(user):
    """Returns whether the user is using 2fa or not."""
    return default_device(user)


def using_two_factor(user):
    """Returns whether the user is using 2fa or not (Cached)."""
    if hasattr(user, "using_two_factor_cache"):
        # First try local memory, as we call this a lot in one request
        return user.using_two_factor_cache
    cache_key = two_factor_cache_key(user)
    val = cache.get(cache_key)
    if val is not None:
        # If local memory failed, but we got it from memcache, set local memory
        user.using_two_factor_cache = val
        return val
    val = bool(_using_two_factor(user))

    # We didn't find it in the cache, so set it there and local memory
    cache.set(cache_key, val, None)  # Cache forever
    user.using_two_factor_cache = val
    return val


def field_exists(model, field):
    try:
        field = model._meta.get_field(field)
    except FieldDoesNotExist:
        return False
    return True


class LoginRequiredNoErrorMixin(LoginRequiredMixin):
    """
    Overwrites Django's 2.2 LoginRequiredMixin so as to not raise an error and
    redirect instead.
    """

    def handle_no_permission(self):
        return redirect_to_login(
            self.request.get_full_path(),
            self.get_login_url(),
            self.get_redirect_field_name(),
        )



from common.helpers.data_migration_loader import load_data_from_file
from django.db import migrations


class Migration(migrations.Migration):

    dependencies = [("common", "0005_add_worksheets")]

    def dummy_reverse_code(app, schema_editor):
        pass

    operations = [
        migrations.RunPython(
            load_data_from_file("aimmo_characters2.json"),
            reverse_code=dummy_reverse_code,
        )
    ]



# Generated by Django 2.2.26 on 2022-02-10 17:13

from django.db import migrations, models
import django.db.models.deletion
import django.utils.timezone


class Migration(migrations.Migration):

    dependencies = [
        ("common", "0015_dailyactivity"),
    ]

    operations = [
        migrations.CreateModel(
            name="JoinReleaseStudent",
            fields=[
                (
                    "id",
                    models.AutoField(
                        auto_created=True,
                        primary_key=True,
                        serialize=False,
                        verbose_name="ID",
                    ),
                ),
                ("action_type", models.CharField(max_length=64)),
                (
                    "action_time",
                    models.DateTimeField(default=django.utils.timezone.now),
                ),
                (
                    "student",
                    models.ForeignKey(
                        on_delete=django.db.models.deletion.CASCADE,
                        related_name="student",
                        to="common.Student",
                    ),
                ),
            ],
        ),
    ]



from django.conf import settings

# Email address to source notifications from
EMAIL_ADDRESS = getattr(settings, "EMAIL_ADDRESS", "no-reply@codeforlife.education")

# Dotmailer URLs for adding users to the newsletter address book
DOTMAILER_CREATE_CONTACT_URL = getattr(settings, "DOTMAILER_CREATE_CONTACT_URL", "")
DOTMAILER_MAIN_ADDRESS_BOOK_URL = getattr(
    settings, "DOTMAILER_MAIN_ADDRESS_BOOK_URL", ""
)
DOTMAILER_TEACHER_ADDRESS_BOOK_URL = getattr(
    settings, "DOTMAILER_TEACHER_ADDRESS_BOOK_URL", ""
)
DOTMAILER_STUDENT_ADDRESS_BOOK_URL = getattr(
    settings, "DOTMAILER_STUDENT_ADDRESS_BOOK_URL", ""
)
DOTMAILER_NO_ACCOUNT_ADDRESS_BOOK_URL = getattr(
    settings, "DOTMAILER_NO_ACCOUNT_ADDRESS_BOOK_URL", ""
)

# Dotmailer username for API authentication
DOTMAILER_USER = getattr(settings, "DOTMAILER_USER", "")

# Dotmailer password for API authentication
DOTMAILER_PASSWORD = getattr(settings, "DOTMAILER_PASSWORD", "")

# Dotmailer default preferences to what users are signed up to
DOTMAILER_DEFAULT_PREFERENCES = getattr(settings, "DOTMAILER_DEFAULT_PREFERENCES", [])

# Dotmailer URL for getting a user by email
DOTMAILER_GET_USER_BY_EMAIL_URL = getattr(
    settings,
    "DOTMAILER_GET_USER_BY_EMAIL_URL",
    "",
)

# Dotmailer URL for deleting a contact by id
DOTMAILER_DELETE_USER_BY_ID_URL = getattr(
    settings,
    "DOTMAILER_DELETE_USER_BY_ID_URL",
    "",
)

# Dotmailer URL for adding consent data to a user
DOTMAILER_PUT_CONSENT_DATA_URL = getattr(settings, "DOTMAILER_PUT_CONSENT_DATA_URL", "")

# Dotmailer URL for sending a triggered campaign to a users
DOTMAILER_SEND_CAMPAIGN_URL = getattr(settings, "DOTMAILER_SEND_CAMPAIGN_URL", "")

# ID of the "Thanks for staying!" campaign in Dotmailer
DOTMAILER_THANKS_FOR_STAYING_CAMPAIGN_ID = getattr(
    settings, "DOTMAILER_THANKS_FOR_STAYING_CAMPAIGN_ID", ""
)

# The name of the google app engine service the application is running on, local otherwise
MODULE_NAME = getattr(settings, "MODULE_NAME", "local")

# Boolean indicating if OneTrust cookie management is enabled or not
COOKIE_MANAGEMENT_ENABLED = getattr(settings, "COOKIE_MANAGEMENT_ENABLED", True)


def domain():
    """Returns the full domain depending on whether it's local, dev, staging or prod."""
    domain = "https://www.codeforlife.education"

    if MODULE_NAME == "local":
        domain = "localhost:8000"
    elif MODULE_NAME == "staging" or MODULE_NAME == "dev":
        domain = f"https://{MODULE_NAME}-dot-decent-digit-629.appspot.com"

    return domain



import random
import sys

from common.helpers.emails import generate_token
from common.models import Teacher
from django.core import mail

from . import email


def generate_details(**kwargs):
    random_int = random.randint(1, sys.maxsize)
    first_name = kwargs.get("first_name", "Test")
    last_name = kwargs.get("last_name", f"Teacher {random_int}")
    email_address = kwargs.get("email_address", f"testteacher{random_int}@codeforlife.com")
    password = kwargs.get("password", "Password2!")

    return first_name, last_name, email_address, password


def signup_teacher_directly(preverified=True, **kwargs):
    """
    Creates a Teacher object by using the details passed in as kwargs, or by
    generating random details. Also verifies the teacher's email if preverified
    is True.
    :param preverified: whether or not the teacher's email should be verified.
    :return: the teacher's email and password.
    """
    first_name, last_name, email_address, password = generate_details(**kwargs)
    teacher = Teacher.objects.factory(first_name, last_name, email_address, password)
    generate_token(teacher.new_user, preverified=preverified)
    teacher.user.save()
    return email_address, password


def signup_duplicate_teacher_fail(page, duplicate_email):
    page = page.go_to_signup_page()

    first_name, last_name, email_address, password = generate_details()
    page = page.signup(first_name, last_name, duplicate_email, password, password)

    page = page.return_to_home_page()

    page = email.follow_duplicate_account_link_to_login(page, mail.outbox[0], "teacher")
    mail.outbox = []

    return page, email_address, password


def signup_teacher(page, newsletter=False):
    page = page.go_to_signup_page()

    first_name, last_name, email_address, password = generate_details()
    page = page.signup(
        first_name, last_name, email_address, password=password, confirm_password=password, newsletter=newsletter
    )

    page = page.return_to_home_page()

    page = email.follow_verify_email_link_to_onboarding(page, mail.outbox[0])
    mail.outbox = []

    return page, email_address, password


def verify_email(page):
    assert len(mail.outbox) > 0

    page = email.follow_verify_email_link_to_login(page, mail.outbox[0], "teacher")
    mail.outbox = []

    return page



from common.helpers.data_migration_loader import load_data_from_file
from django.db import migrations


class Migration(migrations.Migration):

    dependencies = [
        ("common", "0007_add_pdf_names_to_first_two_worksheets"),
    ]

    operations = [
        migrations.RunPython(
            migrations.RunPython.noop, reverse_code=migrations.RunPython.noop
        )
    ]



# Generated by Django 3.2.12 on 2022-04-07 16:46

from django.db import migrations, models


class Migration(migrations.Migration):

    dependencies = [
        ("common", "0019_aimmocharacter_alt"),
    ]

    operations = [
        migrations.AddField(
            model_name="class",
            name="is_active",
            field=models.BooleanField(default=True),
        ),
        migrations.AlterField(
            model_name="class",
            name="access_code",
            field=models.CharField(max_length=5, null=True),
        ),
    ]



import re
from datetime import timedelta
from uuid import uuid4

from django.contrib.auth.models import User
from django.db import models
from django.utils import timezone
from django_countries.fields import CountryField


class UserProfile(models.Model):
    user = models.OneToOneField(User, on_delete=models.CASCADE)
    developer = models.BooleanField(default=False)

    awaiting_email_verification = models.BooleanField(default=False)

    # Holds the user's earned kurono badges. This information has to be on the UserProfile as the Avatar objects are
    # deleted every time the Game gets deleted.
    # This is a string showing which badges in which worksheets have been earned. The format is "X:Y" where X is the
    # worksheet ID and Y is the badge ID. This repeats for all badges and each pair is comma-separated.
    aimmo_badges = models.CharField(max_length=200, null=True, blank=True)

    def __str__(self):
        return f"{self.user.first_name} {self.user.last_name}"

    def joined_recently(self):
        now = timezone.now()
        return now - timedelta(days=7) <= self.user.date_joined


class EmailVerification(models.Model):
    user = models.ForeignKey(User, related_name="email_verifications", null=True, blank=True, on_delete=models.CASCADE)
    token = models.CharField(max_length=30)
    email = models.CharField(max_length=200, null=True, default=None, blank=True)
    expiry = models.DateTimeField()
    verified = models.BooleanField(default=False)

    def __str__(self):
        return f"Email verification for {self.user.username}, ({self.email})"


class SchoolModelManager(models.Manager):
    # Filter out inactive schools by default
    def get_queryset(self):
        return super().get_queryset().filter(is_active=True)


class School(models.Model):
    name = models.CharField(max_length=200)
    postcode = models.CharField(max_length=10, null=True)
    country = CountryField(blank_label="(select country)")
    creation_time = models.DateTimeField(default=timezone.now, null=True)
    is_active = models.BooleanField(default=True)

    objects = SchoolModelManager()

    def __str__(self):
        return self.name

    def classes(self):
        teachers = self.teacher_school.all()
        if teachers:
            classes = []
            for teacher in teachers:
                if teacher.class_teacher.all():
                    classes.extend(list(teacher.class_teacher.all()))
            return classes
        return None

    def admins(self):
        teachers = self.teacher_school.all()
        return [teacher for teacher in teachers if teacher.is_admin] if teachers else None

    def anonymise(self):
        self.name = uuid4().hex
        self.postcode = ""
        self.is_active = False
        self.save()


class TeacherModelManager(models.Manager):
    def factory(self, first_name, last_name, email, password):
        user = User.objects.create_user(
            username=email, email=email, password=password, first_name=first_name, last_name=last_name
        )

        user_profile = UserProfile.objects.create(user=user)

        return Teacher.objects.create(user=user_profile, new_user=user)

    # Filter out non active teachers by default
    def get_queryset(self):
        return super().get_queryset().filter(new_user__is_active=True)


class Teacher(models.Model):
    user = models.OneToOneField(UserProfile, on_delete=models.CASCADE)
    new_user = models.OneToOneField(User, related_name="new_teacher", null=True, blank=True, on_delete=models.CASCADE)
    school = models.ForeignKey(School, related_name="teacher_school", null=True, on_delete=models.SET_NULL)
    is_admin = models.BooleanField(default=False)
    blocked_time = models.DateTimeField(null=True, blank=True)
    invited_by = models.ForeignKey(
        "self", related_name="invited_teachers", null=True, blank=True, on_delete=models.SET_NULL
    )

    objects = TeacherModelManager()

    def teaches(self, userprofile):
        if hasattr(userprofile, "student"):
            student = userprofile.student
            return not student.is_independent() and student.class_field.teacher == self

    def has_school(self):
        return self.school is not (None or "")

    def has_class(self):
        return self.class_teacher.exists()

    def __str__(self):
        return f"{self.new_user.first_name} {self.new_user.last_name}"


class SchoolTeacherInvitationModelManager(models.Manager):
    # Filter out inactive invitations by default
    def get_queryset(self):
        return super().get_queryset().filter(is_active=True)


class SchoolTeacherInvitation(models.Model):
    token = models.CharField(max_length=32)
    school = models.ForeignKey(School, related_name="teacher_invitations", null=True, on_delete=models.SET_NULL)
    from_teacher = models.ForeignKey(Teacher, related_name="school_invitations", null=True, on_delete=models.SET_NULL)
    invited_teacher_first_name = models.CharField(max_length=150)  # Same as User model
    invited_teacher_last_name = models.CharField(max_length=150)  # Same as User model
    invited_teacher_email = models.EmailField()  # Same as User model
    invited_teacher_is_admin = models.BooleanField(default=False)
    expiry = models.DateTimeField()
    creation_time = models.DateTimeField(default=timezone.now, null=True)
    is_active = models.BooleanField(default=True)

    objects = SchoolTeacherInvitationModelManager()

    @property
    def is_expired(self):
        return self.expiry < timezone.now()

    def __str__(self):
        return f"School teacher invitation for {self.invited_teacher_email} to {self.school.name}"

    def anonymise(self):
        self.invited_teacher_first_name = uuid4().hex
        self.invited_teacher_last_name = uuid4().hex
        self.invited_teacher_email = uuid4().hex
        self.is_active = False
        self.save()


class ClassModelManager(models.Manager):
    def all_members(self, user):
        members = []
        if hasattr(user, "teacher"):
            members.append(user.teacher)
            if user.teacher.has_school():
                classes = user.teacher.class_teacher.all()
                for c in classes:
                    members.extend(c.students.all())
        else:
            c = user.student.class_field
            members.append(c.teacher)
            members.extend(c.students.all())
        return members

    # Filter out non active classes by default
    def get_queryset(self):
        return super().get_queryset().filter(is_active=True)


class Class(models.Model):
    name = models.CharField(max_length=200)
    teacher = models.ForeignKey(Teacher, related_name="class_teacher", on_delete=models.CASCADE)
    access_code = models.CharField(max_length=5, null=True)
    classmates_data_viewable = models.BooleanField(default=False)
    always_accept_requests = models.BooleanField(default=False)
    accept_requests_until = models.DateTimeField(null=True)
    creation_time = models.DateTimeField(default=timezone.now, null=True)
    is_active = models.BooleanField(default=True)
    created_by = models.ForeignKey(
        Teacher, null=True, blank=True, related_name="created_classes", on_delete=models.SET_NULL
    )

    objects = ClassModelManager()

    def __str__(self):
        return self.name

    @property
    def active_game(self):
        games = self.game_set.filter(game_class=self, is_archived=False)
        if len(games) >= 1:
            assert len(games) == 1  # there should NOT be more than one active game
            return games[0]
        return None

    def has_students(self):
        students = self.students.all()
        return students.count() != 0

    def get_requests_message(self):
        if self.always_accept_requests:
            external_requests_message = "This class is currently set to always accept requests."
        elif self.accept_requests_until is not None and (self.accept_requests_until - timezone.now()) >= timedelta():
            external_requests_message = (
                "This class is accepting external requests until "
                + self.accept_requests_until.strftime("%d-%m-%Y %H:%M")
                + " "
                + timezone.get_current_timezone_name()
            )
        else:
            external_requests_message = "This class is not currently accepting external requests."

        return external_requests_message

    def anonymise(self):
        self.name = uuid4().hex
        self.access_code = ""
        self.is_active = False
        self.save()

        # Remove independent students' requests to join this class
        self.class_request.clear()

    class Meta(object):
        verbose_name_plural = "classes"


class UserSession(models.Model):
    user = models.ForeignKey(User, on_delete=models.CASCADE)
    login_time = models.DateTimeField(default=timezone.now)
    school = models.ForeignKey(School, null=True, on_delete=models.SET_NULL)
    class_field = models.ForeignKey(Class, null=True, on_delete=models.SET_NULL)
    login_type = models.CharField(max_length=100, null=True)  # for student login

    def __str__(self):
        return f"{self.user} login: {self.login_time} type: {self.login_type}"


class StudentModelManager(models.Manager):
    def get_random_username(self):
        while True:
            random_username = uuid4().hex[:30]  # generate a random username
            if not User.objects.filter(username=random_username).exists():
                return random_username

    def schoolFactory(self, klass, name, password, login_id=None):
        user = User.objects.create_user(username=self.get_random_username(), password=password, first_name=name)
        user_profile = UserProfile.objects.create(user=user)

        return Student.objects.create(class_field=klass, user=user_profile, new_user=user, login_id=login_id)

    def independentStudentFactory(self, name, email, password):
        user = User.objects.create_user(username=email, email=email, password=password, first_name=name)

        user_profile = UserProfile.objects.create(user=user)

        return Student.objects.create(user=user_profile, new_user=user)


class Student(models.Model):
    class_field = models.ForeignKey(Class, related_name="students", null=True, on_delete=models.CASCADE)
    # hashed uuid used for the unique direct login url
    login_id = models.CharField(max_length=64, null=True)
    user = models.OneToOneField(UserProfile, on_delete=models.CASCADE)
    new_user = models.OneToOneField(User, related_name="new_student", null=True, blank=True, on_delete=models.CASCADE)
    pending_class_request = models.ForeignKey(Class, related_name="class_request", null=True, on_delete=models.SET_NULL)
    blocked_time = models.DateTimeField(null=True)

    objects = StudentModelManager()

    def is_independent(self):
        return not self.class_field

    def __str__(self):
        return f"{self.new_user.first_name} {self.new_user.last_name}"


def stripStudentName(name):
    return re.sub("[ \t]+", " ", name.strip())


class AimmoCharacterManager(models.Manager):
    def sorted(self):
        return self.get_queryset().order_by("sort_order")


class AimmoCharacter(models.Model):
    name = models.CharField(max_length=255)
    description = models.TextField()
    image_path = models.CharField(max_length=255)
    sort_order = models.IntegerField()
    alt = models.CharField(max_length=255, null=True)
    objects = AimmoCharacterManager()

    def __str__(self) -> str:
        return self.name


# -----------------------------------------------------------------------
# Below are models used for data tracking and maintenance
# -----------------------------------------------------------------------
class JoinReleaseStudent(models.Model):
    """
    To keep track when a student is released to be independent student or
    joins a class to be a school student.
    """

    JOIN = "join"
    RELEASE = "release"

    student = models.ForeignKey(Student, related_name="student", on_delete=models.CASCADE)
    # either "release" or "join"
    action_type = models.CharField(max_length=64)
    action_time = models.DateTimeField(default=timezone.now)


class DailyActivity(models.Model):
    """
    A model to record sets of daily activity. Currently used to record the amount of
    student details download clicks, through the CSV and login cards methods, per day.
    """

    date = models.DateField(default=timezone.now)
    csv_click_count = models.PositiveIntegerField(default=0)
    login_cards_click_count = models.PositiveIntegerField(default=0)
    primary_coding_club_downloads = models.PositiveIntegerField(default=0)
    python_coding_club_downloads = models.PositiveIntegerField(default=0)

    def __str__(self):
        return f"Activity on {self.date}: CSV clicks: {self.csv_click_count}, login cards clicks: {self.login_cards_click_count}"


class DynamicElement(models.Model):
    """
    This model is meant to allow us to quickly update some elements dynamically on the website without having to
    redeploy everytime. For example, if a maintenance banner needs to be added, we check the box in the Django admin
    panel, edit the text and it'll show immediately on the website.
    """

    name = models.CharField(max_length=64, unique=True, editable=False)
    active = models.BooleanField(default=False)
    text = models.TextField(null=True, blank=True)

    def __str__(self) -> str:
        return self.name



# Generated by Django 3.2.15 on 2022-09-08 15:27

from django.db import migrations, models


class Migration(migrations.Migration):

    dependencies = [
        ('common', '0027_class_created_by'),
    ]

    operations = [
        migrations.AddField(
            model_name='dailyactivity',
            name='primary_coding_club_downloads',
            field=models.PositiveIntegerField(default=0),
        ),
        migrations.AddField(
            model_name='dailyactivity',
            name='python_coding_club_downloads',
            field=models.PositiveIntegerField(default=0),
        ),
    ]
